@@ -50,7 +50,10 @@ sub new ($%) {
 	$vbox->pack_start($toolbar, FALSE, FALSE, 0) if defined $toolbar;
 
 	my $main_widget = $self->main_widget();
-	$vbox->pack_start($main_widget, TRUE, TRUE, 0) if defined $main_widget;
+	if (defined $main_widget) {
+		$self->set_focus_child($main_widget);
+		$vbox->pack_start($main_widget, TRUE, TRUE, 0);
+	}
 
 	my $statusbar = $self->statusbar();
 	$vbox->pack_start($statusbar, FALSE, FALSE, 0) if defined $statusbar;
@@ -211,6 +214,8 @@ sub statusbar ($) {
 
 	if (! exists $self->{statusbar}) {
 		$self->{statusbar} = Gtk2::Statusbar->new();
+		my $label = (($self->{statusbar}->get_children)[0]->get_children)[0];
+		$label->set_selectable(1) if $label->isa('Gtk2::Label');
 	}
 
 	return $self->{statusbar};
@@ -235,6 +240,21 @@ sub set_widget_sensitivity ($$@) {
 	}
 }
 
+sub set_deep_widget_focus_child ($$@) {
+   my $self = shift;
+   my $widget = shift;
+	my @idxs = @_;
+	my $default_idx = $idxs[-1] || 0;
+
+	while ($widget->isa('Gtk2::Container')) {
+		my $idx = @idxs? shift @idxs: $default_idx;
+		my $child = ($widget->get_children)[$idx];
+		last unless $child;
+		$widget->set_focus_child($child);
+		$widget = $child;
+	}
+} 
+
 sub show_about ($;$) {
 	my $self = shift;
 	my $all = shift;
