@@ -200,9 +200,19 @@ sub show_string ($$) {
 
 	if (-e $arg) {
 		if (-d "$arg/{arch}") {
-			$self->show(
-				Arch::Tree->new($arg)->get_changeset($self->temp->dir_name)
-			);
+			my $cset =
+				Arch::Tree->new($arg)->get_changeset($self->temp->dir_name);
+
+			if (! defined $cset) {
+				$self->alert(
+					"You may be using <b>untagged-source unrecognized</b> "
+						. "and have untagged source files in your tree. "
+						. "Please add file ids or remove the offending files.",
+					"Could not get local tree changes"
+				);
+			} else {
+				$self->show($cset);
+			}
 
 		} elsif (-f "$arg/mod-dirs-index") {
 			$self->show(
@@ -218,9 +228,9 @@ sub show_string ($$) {
 
 	} else {
 		eval {
-			$self->show(
-				$self->arch->get_revision_changeset($arg, $self->temp->dir_name)
-			);
+			$self->show($self->arch->get_revision_changeset(
+					$arg, $self->temp->dir_name
+			));
 		};
 		$self->alert(
 			"Could not fetch changeset for revision <b>$arg</b>.",
