@@ -29,12 +29,16 @@ use Arch::Inventory qw(:category :type :id_type);
 use Arch::Changes qw(:type);
 use Arch::Util qw(load_file save_file);
 
+use Arch::FileHighlighter;
+
 use ArchWay::Widget::Inventory::Tree;
 use ArchWay::Widget::Inventory::Directory;
 use ArchWay::Widget::Inventory::Entry;
 
 use ArchWay::Widget::FrameScroll;
 use ArchWay::Widget::ChangeList;
+use ArchWay::Widget::Text;
+use ArchWay::Widget::Diff;
 use ArchWay::Widget::Log;
 
 use ArchWay::MainWindow::Revisions;
@@ -44,29 +48,19 @@ sub init ($) {
 
 	$self->SUPER::init;
 
-	# menus (name, stock id, label)
-	my @menus = (
+	# define action groups
+	my $tree_actions = Gtk2::ActionGroup->new("TreeActions");
+	$tree_actions->add_actions([
 		[ "TreeMenu", undef, "_Tree" ],
 		[ "FileMenu", undef, "_File" ],
-	);
+	], undef);
 
-	# items (name, stock id, label, accelerator, tooltip, callback)
-	my @items = (
+	$tree_actions->add_actions([
 		[
 			"Refresh", "gtk-refresh", "_Refresh",
 			undef, "Refresh inventory",
 			sub { $self->refresh },
 		],
-		[
-			"Undo", "gtk-undo", "_Undo",
-			undef, "Undo",
-			sub {}
-		],
-		[
-			"Redo", "gtk-redo", "_Redo",
-			undef, "Redo",
-		 	sub {}
-		],
 		[
 			"Commit", "gtk-apply", "_Commit",
 			undef, "Commit",
@@ -83,41 +77,54 @@ sub init ($) {
 			sub { $self->show_changes }
 		],
 		[
-			"Ancestry", undef, "_Ancestry",
+			"Ancestry", "gtk-paste", "_Ancestry",
 			undef, "View project tree ancestry",
 			sub { $self->show_ancestry }
 		],
+	], undef);
+
+	my $file_actions = Gtk2::ActionGroup->new("FileActions");
+	$file_actions->add_actions([
 		[
-			"Diff", "gtk-search", "_View Diff",
-			undef, "View file diff",
-			sub {}
-		],
-		[
-			"History", undef, "_History",
-			undef, "View File History",
-			sub { $self->show_file_history }
-		],
-		[
-			"Add", "gtk-add", "_Add",
-			undef, "Add an explicit inventory id to the selected file",
-			sub { $self->add_file }
-		],
-		[
-			"Move", "gtk-copy", "_Move",
-			undef, "Move/Rename the selected file",
-			sub {}
-		],
-		[
-			"Delete", "gtk-remove", "_Delete",
-			undef, "Delete the selected files explicit inventory id, keeping the file",
-			sub { $self->delete_file }
+			"View", "gtk-edit", "_View",
+			undef, "View selected file",
+			sub { $self->show_file }
 		],
 		[
 			"Remove", "gtk-delete", "_Remove",
 			undef, "Remove the selected file",
 			sub { $self->remove_file }
+		]
+	], undef);
+
+	my $source_actions = Gtk2::ActionGroup->new("SourceActions");
+	$source_actions->add_actions([
+		[
+			"Diff", "gtk-find", "_View Diff",
+			undef, "View file diff",
+			sub { $self->show_file_diff }
 		],
-	);
+		[
+			"History", "gtk-paste", "_History",
+			undef, "View File History",
+			sub { $self->show_file_history }
+		],
+	], undef);
+
+	my $non_source_actions = Gtk2::ActionGroup->new("NonSourceActions");
+	$non_source_actions->add_actions([
+		[
+			"Add", "gtk-add", "_Add",
+			undef, "Add an explicit inventory id to the selected file",
+			sub { $self->add_file }
+		],
+	], undef);
+
+	# add actions
+	$self->ui->insert_action_group($tree_actions, 0);
+	$self->ui->insert_action_group($file_actions, 0);
+	$self->ui->insert_action_group($source_actions, 0);
+	$self->ui->insert_action_group($non_source_actions, 0);
 
 	# menu/toolbar layout
 	my $layout = <<_EOF_;
@@ -125,64 +132,66 @@ sub init ($) {
 	<menubar name="MenuBar">
 		<menu action="FileMenu">
 			<placeholder name="FileMenuItems">
+				<menuitem action="Add" />
+				<menuitem action="Remove" />
+				<separator />
+				<menuitem action="View" />
 				<menuitem action="Diff" />
 				<menuitem action="History" />
-				<separator />
-				<menuitem action="Add" />
-				<menuitem action="Move" />
-				<menuitem action="Delete" />
-				<menuitem action="Remove" />
 			</placeholder>
 		</menu>
 
 		<placeholder name="ApplicationMenus">
 			<menu action="TreeMenu">
-				<menuitem action="Refresh" />
-				<menuitem action="Undo" />
-				<menuitem action="Redo" />
-				<separator />
 				<menuitem action="Ancestry" />
 				<menuitem action="Changes" />
 				<menuitem action="EditLog" />
 				<menuitem action="Commit" />
+				<separator />
+				<menuitem action="Refresh" />
 			</menu>
 		</placeholder>
 	</menubar>
 
 	<toolbar name="ToolBar">
 		<toolitem action="Add" />
-		<toolitem action="Delete" />
 		<toolitem action="Remove" />
 		<separator />
+		<toolitem action="View" />
+		<toolitem action="Diff" />
+		<toolitem action="History" />
+		<separator />
+		<toolitem action="Ancestry" />
 		<toolitem action="Changes" />
 		<toolitem action="EditLog" />
 		<toolitem action="Commit" />
 		<separator />
 		<toolitem action="Refresh" />
 	</toolbar>
+
+	<popup name="Popup">
+		<menuitem action="Add" />
+		<menuitem action="Remove" />
+		<separator />
+		<menuitem action="View" />
+		<menuitem action="Diff" />
+		<menuitem action="History" />
+	</popup>
 </ui>
 _EOF_
 
-	# add menu/toolbar to ui
-	my $actions = Gtk2::ActionGroup->new("TreeActions");
-	$actions->add_actions(\@menus, undef);
-	$actions->add_actions(\@items, undef);
-
-	$self->ui->insert_action_group($actions, 0);
 	$self->ui->add_ui_from_string($layout);
 
-	# deactivate unimplemented widgets
-	$self->set_widget_sensitivity(
-		FALSE,
-		qw(
-			/MenuBar/FileMenu/FileMenuItems/Diff
-			/MenuBar/FileMenu/FileMenuItems/Move
-			/MenuBar/ApplicationMenus/TreeMenu/Undo
-			/MenuBar/ApplicationMenus/TreeMenu/Redo
-		),
-	);
+	# directory view popup
+	$self->file_view->signal_connect('button-press-event', sub {
+		my $ev = $_[1];
+
+		$self->ui->get_widget('/Popup')->popup(
+			undef, undef, undef, undef,
+			$ev->button, $ev->time
+		) if $ev->button == 3;
+	});
 
-	#init ui
 	$self->set_default_size(800, 600);
 	$self->refresh;
 }
@@ -208,6 +217,9 @@ sub main_widget ($) {
 		$vbox->pack_start($self->version_label, FALSE, FALSE, 3);
 		$vbox->pack_start($hpaned, TRUE, TRUE, 0);
 		$vbox->pack_start($self->entry_view, FALSE, FALSE, 0);
+
+		$vbox->set_focus_child($hpaned);
+		$self->set_deep_widget_focus_child($hpaned, 1);
 		
 		$self->{main_widget} = $vbox;
 	}
@@ -288,19 +300,31 @@ sub refresh ($) {
 	$self->{inventory}->annotate_fs;
 
 	# annotate inventory tree with changes
-	foreach my $change ($self->tree->get_changes->get) {
-		my $path = $change->{type} eq RENAME ?
-			$change->{arguments}->[1] : $change->{arguments}->[0];
-
-		# skip arch internal files
-		next if ($path =~ m!^{arch}(/|$)!);
-		next if ($path =~ m!(^|/)\.arch-ids(/|$)!);
-
-		my $entry = $self->inventory->get_entry($path);
-		if ($entry) {
-			$entry->{changes}->{$change->{type}} = $change;
-		} else {
-			warn "cannot annotate changes for $path\n";
+	my $changes = $self->tree->get_changes;
+	if (! defined $changes) {
+		$self->alert(
+			"You may be using <b>untagged-source unrecognized</b> "
+				. "and have untagged source files in your tree. Please add "
+				. "file ids or remove the offending files, and press the "
+				. "<b>Refresh</b> button.",
+			"Could not get local tree changes"
+		);
+	} else {
+	 	foreach my $change ($changes->get) {
+	 		my $path = $change->{type} eq RENAME ?
+	 			$change->{arguments}->[1] : $change->{arguments}->[0];
+	 
+	 		# skip arch internal files
+	 		next if $path =~ m!^{arch}(/|$)!;
+	 		next if $path =~ m!(^|/)\.arch-ids(/|$)!;
+	 		next if $path =~ m!(^|/)\.arch-inventory$!;
+	 
+	 		my $entry = $self->inventory->get_entry($path);
+	 		if ($entry) {
+	 			$entry->{changes}->{$change->{type}} = $change;
+	 		} else {
+	 			warn "cannot annotate changes for $path\n";
+	 		}
 		}
 	}
 
@@ -337,33 +361,16 @@ sub update_sensitivity ($) {
 	my $path  = $self->file_view->get_selected_path;
 	my $entry = defined $path ? $self->inventory->get_entry($path) : undef;
 
-	$self->set_widget_sensitivity(
-		defined $entry && $entry->{path},
-		qw(
-			/MenuBar/FileMenu/FileMenuItems/History
-		),
-	);
-	$self->set_widget_sensitivity(
-		defined $entry && $entry->{untagged},
-		qw(
-			/MenuBar/FileMenu/FileMenuItems/Add
-			/ToolBar/Add
-		),
-	);
-	$self->set_widget_sensitivity(
-		defined $entry && ($entry->{category} eq SOURCE) && ($entry->{id_type} eq EXPLICIT),
-		qw(
-			/MenuBar/FileMenu/FileMenuItems/Delete
-			/ToolBar/Delete
-		),
-	);
-	$self->set_widget_sensitivity(
-		defined $entry && $entry->{path},
-		qw(
-			/MenuBar/FileMenu/FileMenuItems/Remove
-			/ToolBar/Remove
-		),
-	);
+	foreach my $ag ($self->ui->get_action_groups) {
+		$ag->set_sensitive(defined $entry && $entry->{path})
+			if $ag->get_name eq 'FileActions';
+
+		$ag->set_sensitive(defined $entry && ($entry->{category} eq SOURCE))
+			if $ag->get_name eq 'SourceActions';
+
+		$ag->set_sensitive(defined $entry && $entry->{untagged})
+			if $ag->get_name eq 'NonSourceActions';
+	}
 }
 
 sub update_file_view ($) {
@@ -402,10 +409,123 @@ sub activate_item ($) {
 	}
 }
 
+sub show_file ($) {
+	my $self = shift;
+
+	my $path = $self->file_view->get_selected_path;
+	my $entry = $self->inventory->get_entry($path);
+
+	if ($entry->{type} eq DIRECTORY) {
+		$self->alert('Cannot view a directory.');
+		return;
+	}
+
+	my $fpath = $self->tree->root . "/$path";
+
+	if (-B $fpath) {
+		$self->alert('Cannot view a binary file.');
+		return;
+	}
+
+	my $text_widget = ArchWay::Widget::Text->new;
+	$text_widget->show(
+		${Arch::FileHighlighter->instance->highlight($fpath)}
+	);
+
+	my $dlg = $self->widget_dialog(
+		$path, $text_widget,
+		buttons => [ 'Annotate' => 1, 'gtk-close' => 'close' ]
+	);
+
+	my $annotated = 0; # gotta love closures
+	$dlg->signal_connect('response' => sub {
+		return if $annotated || ($_[1] != 1);
+
+		$annotated = 1;
+
+		my ($lines, $rd_idx, $rd) =
+			$self->tree->get_annotate_revision_descs($path, group => 1);
+
+		my $buffer = $text_widget->get_buffer;
+		my $len = int(log(scalar @$rd) / log(10)) + 6;
+		
+		my $iter = $buffer->get_iter_at_offset(0);
+ 		for (my $b = 0; $b < @$lines; ++$b) {
+ 			for (my $l = 0; $l < @{$lines->[$b]}; ++$l) {
+				my $text = sprintf(
+					'%-*s', $len, $l
+						? ''
+						: 'rev-' . ($rd_idx->[$b] + 1) .':'
+				);
+
+ 				$buffer->insert_with_tags_by_name(
+ 					$iter,
+					$text,
+ 					"annotate_zebra" . $b % 2,
+ 				);
+				$buffer->insert($iter, ' ');
+
+ 				$iter->forward_line;
+ 			}
+ 		}
+
+		for (my $i = 0; $i < @$rd; ++$i) {
+			$rd->[$i]->{name} = 'rev-' . ($i + 1) . " ($rd->[$i]->{name})";
+		}
+
+		# modify dialog
+		my $text_scroll = ($dlg->vbox->get_children)[0];
+		$dlg->vbox->remove($text_scroll);
+
+		my $rev_widget = ArchWay::Widget::Revisions->new;
+		$rev_widget->add_revision_descs($rd);
+
+		my $rev_scroll =
+			ArchWay::Widget::FrameScroll->new($rev_widget, undef, 1);
+
+		my $vpaned = Gtk2::VPaned->new;
+		$vpaned->pack1($text_scroll, TRUE, TRUE);
+		$vpaned->pack2($rev_scroll, TRUE, TRUE);
+		$vpaned->set_position(400);
+
+		$dlg->vbox->pack_start($vpaned, TRUE, TRUE, 0);
+		$dlg->show_all;
+ 	});
+
+	$dlg->set_default_size(700, 600);
+	$dlg->show_all;
+}
+
+sub show_file_diff ($) {
+	my $self = shift;
+
+	my $path = $self->file_view->get_selected_path;
+	my $entry = $self->inventory->get_entry($path);
+
+	if ($entry->{type} eq DIRECTORY) {
+		$self->alert('Cannot run file-diff on a directory.');
+		return;
+	}
+
+	my $diff_widget = ArchWay::Widget::Diff->new;
+	$diff_widget->show($self->tree->get_file_diff($path));
+
+	my $dlg = $self->widget_dialog("Diff for $path", $diff_widget);
+	$dlg->set_default_size(500, 600);
+	$dlg->show_all;
+}
+
 sub show_file_history ($) {
 	my $self = shift;
 
 	my $path = $self->file_view->get_selected_path;
+	my $entry = $self->inventory->get_entry($path);
+
+	if ($entry->{type} eq DIRECTORY) {
+		$self->alert('Cannot run file history on a directory.');
+		return;
+	}
+
 	my $descs = [ reverse @{$self->tree->get_ancestry_revision_descs($path)} ];
 
 	my $dlg = $self->session->activate_gui('revs', "File history for $path");
@@ -440,61 +560,26 @@ sub add_file ($) {
 	}
 }
 
-sub delete_file ($) {
-	my $self = shift;
-
-	my $path = $self->file_view->get_selected_path;
-	my $entry = $self->inventory->get_entry($path);
-
-	if ($self->tree->delete($path) == 0) {
-		$entry->{category} = PRECIOUS;
-		$entry->{untagged} = 1;
-		$entry->{id}       = undef;
-		$entry->{changes}  = {
-			REMOVE() => {
-				type      => REMOVE,
-				is_dir    => $entry->{type} eq DIRECTORY,
-				arguments => [ $path ],
-			}
-		};
-		
-		# alternative: mark entire subtree as precious/removed
-		if ($entry->{type} eq DIRECTORY) {
-			my $tpath = $self->tree_view->get_selected_path;
-			
-			$entry->{children} = {};
-			$self->tree_view->show($self->inventory->get_root_entry);
-			$self->tree_view->select_by_path($tpath);
-
-		} else {
-			$self->update_file_view;
-			$self->file_view->select_by_path($path);
-		}
-
-	} else {
-		$self->set_status('Error: delete-id failed for ' . $path);
-	}
-}
-
 sub remove_file ($) {
 	my $self = shift;
 
 	my $path = $self->file_view->get_selected_path;
 	my $entry = $self->inventory->get_entry($path);
 
-	(my $name = $path) =~ s!^.*/!!;
+	(my $name  = $path) =~ s!^.*/!!;
 	(my $ppath = $path) =~ s!(^|/)[^/]*$!!;
 	my $parent = $self->inventory->get_entry($ppath);
 
 	if ($self->confirm("Really remove $path?")) {
-		if ($entry->{category} eq SOURCE) {
-			$self->tree->delete($path) == 0 or
-				$self->set_status('Error: delete-id failed for ' . $path);
+		if ($entry->{type} eq DIRECTORY) {
+			system('rm', '-r', $self->tree->root . "/$path");
+		} else {
+			my $id   = $self->tree->root . "/$ppath/.arch-ids/$name.id";
+			my $file = $self->tree->root . "/$path";
+			unlink($id)	if -f $id;
+			unlink($file) && delete $parent->{children}->{$name};
 		}
 
-		system('rm', '-r', $self->tree->root . '/' . $path);
-		delete $parent->{children}->{$name};
-
 		$self->tree_view->show($self->inventory->get_root_entry);
 		$self->tree_view->select_by_path($ppath);
 	}
@@ -526,8 +611,9 @@ sub edit_log {
 	my $log_widget = ArchWay::Widget::Log->new;
 	$log_widget->show(load_file($log));
 
-	$self->{edit_log} = my $dlg = $self->create_dialog(
+	$self->{edit_log} = my $dlg = $self->widget_dialog(
 		'Commit Log',
+		$log_widget,
 		buttons => [ qw(
 			gtk-save   accept
 			gtk-cancel reject
@@ -542,11 +628,7 @@ sub edit_log {
 		$self->{edit_log} = undef;
 	});
 
-	my $scroll = ArchWay::Widget::FrameScroll->new($log_widget, undef, 1);
-
-	$dlg->vbox->pack_start($scroll, TRUE, TRUE, 0);
-	$dlg->set_default_size(400, 300);
-
+	$dlg->set_default_size(500, 300);
 	$dlg->show_all;
 }
 
@@ -573,7 +655,6 @@ sub commit {
 	my $chg_widget = ArchWay::Widget::ChangeList->new;
 	$chg_widget->show($tree->get_changes);
 
-
 	my $chg_frame = ArchWay::Widget::FrameScroll->new(
 		$chg_widget, 'Changes', 1
 	);
@@ -593,8 +674,6 @@ sub commit {
 	# dialog
 	$self->{commit} = my $dlg = $self->create_dialog('Commit');
 	$dlg->vbox->pack_start($vpaned, TRUE, TRUE, 0);
-	$dlg->set_default_size(400, 500);
-	$dlg->show_all;
 
 	$dlg->signal_connect('response' => sub {
 		my $archlog = Arch::Log->new($log_widget->get_text);
@@ -618,6 +697,9 @@ sub commit {
 		$dlg->destroy;
 		$self->{commit} = undef;
 	});
+
+	$dlg->set_default_size(500, 500);
+	$dlg->show_all;
 }
 
 1;
