@@ -21,14 +21,13 @@ use warnings;
 package ArchWay::Widget::Inventory::Directory;
 
 use Glib qw(TRUE FALSE);
-use Gtk2;
-
-use base qw(Gtk2::TreeView);
 
 use Arch::Inventory qw(:category :type);
 use Arch::Changes qw(:type);
 use POSIX qw(strftime);
 
+use base qw(ArchWay::Widget::Tree);
+
 use constant MODEL_COL_TYPES => qw(
 	Glib::String
 	Glib::String
@@ -37,7 +36,12 @@ use constant MODEL_COL_TYPES => qw(
 	Glib::String
 	Glib::String
 	Glib::String
+
 	Glib::String
+	Glib::UInt
+	Glib::UInt
+
+	Glib::Scalar
 );
 
 use constant MODEL_COL_TYPE     => 0;
@@ -47,69 +51,10 @@ use constant MODEL_COL_STATUS   => 3;
 use constant MODEL_COL_SIZE     => 4;
 use constant MODEL_COL_MODE     => 5;
 use constant MODEL_COL_MODIFIED => 6;
-use constant MODEL_COL_PATH     => 7;
-
-my @VIEW_COLUMNS = (
-	{
-		title  => 'Filename',
-		renderers => [
-			{
-				class   => 'Gtk2::CellRendererPixbuf',
-				static  => {},
-				dynamic => { stock_id => MODEL_COL_TYPE },
-			},
-			{
-				class   => 'Gtk2::CellRendererText',
-				static  => {},
-				dynamic => { text => MODEL_COL_NAME },
-			},
-		],
-		sort_column => MODEL_COL_NAME,
-	},
-	{
-		title  => 'Category',
-		renderer => {
-			class   => 'Gtk2::CellRendererText',
-			static  => {},
-			dynamic => { text => MODEL_COL_CATEGORY },
-		},
-		sort_column => MODEL_COL_CATEGORY,
-	},
-	{
-		title  => 'Status',
-		renderer => {
-			class   => 'Gtk2::CellRendererText',
-			static  => {},
-			dynamic => { text => MODEL_COL_STATUS },
-		},
-		sort_column => MODEL_COL_STATUS,
-	},
-	{
-		title  => 'Size',
-		renderer => {
-			class   => 'Gtk2::CellRendererText',
-			static  => { xalign => 1.0 },
-			dynamic => { markup => MODEL_COL_SIZE },
-		},
-		sort_column => MODEL_COL_SIZE,
-	},
-	{
-		title  => 'Mode',
-		renderer => {
-			class   => 'Gtk2::CellRendererText',
-			static  => {},
-			dynamic => { markup => MODEL_COL_MODE },
-		},
-	},
-	{
-		title  => 'Last Modified',
-		renderer => {
-			class   => 'Gtk2::CellRendererText',
-			static  => {},
-			dynamic => { markup => MODEL_COL_MODIFIED },
-		},
-	}
-);
+use constant MODEL_COL_SORT_NAME => 7;
+use constant MODEL_COL_SORT_SIZE => 8;
+use constant MODEL_COL_SORT_MODIFIED => 9;
+use constant MODEL_COL_ENTRY    => 10;
 
 my %CATEGORY_NAMES = (
 	TREE()         => 'Arch Tree Root',
@@ -120,90 +65,94 @@ my %CATEGORY_NAMES = (
 	UNRECOGNIZED() => 'Unrecognized',
 );
 
-my %TYPE_NAMES = (
-	FILE()      => 'F',
-	DIRECTORY() => 'D',
-	SYMLINK()   => 'L',
-);
-
 sub new ($$) {
 	my $class     = shift;
 	my $inventory = shift;
 
-	my $self = bless Gtk2::TreeView->new(), $class;
-	$self->set_model(Gtk2::ListStore->new(MODEL_COL_TYPES));
-
-	$self->_init();
+	my $self = $class->SUPER::new(Gtk2::ListStore->new(MODEL_COL_TYPES));
+	$self->add_columns(
+		{
+			title  => 'Filename',
+			renderers => [
+				{
+					class   => 'Gtk2::CellRendererPixbuf',
+					static  => {},
+					dynamic => { stock_id => MODEL_COL_TYPE },
+				},
+				{
+					class   => 'Gtk2::CellRendererText',
+					static  => {},
+					dynamic => { text => MODEL_COL_NAME },
+				},
+			],
+			sort_column => MODEL_COL_SORT_NAME,
+		},
+		{
+			title  => 'Category',
+			renderer => {
+				class   => 'Gtk2::CellRendererText',
+				static  => {},
+				dynamic => { text => MODEL_COL_CATEGORY },
+			},
+			sort_column => MODEL_COL_CATEGORY,
+		},
+		{
+			title  => 'Status',
+			renderer => {
+				class   => 'Gtk2::CellRendererText',
+				static  => {},
+				dynamic => { text => MODEL_COL_STATUS },
+			},
+			sort_column => MODEL_COL_STATUS,
+		},
+		{
+			title  => 'Size',
+			renderer => {
+				class   => 'Gtk2::CellRendererText',
+				static  => { xalign => 1.0 },
+				dynamic => { markup => MODEL_COL_SIZE },
+			},
+			sort_column => MODEL_COL_SORT_SIZE,
+		},
+		{
+			title  => 'Mode',
+			renderer => {
+				class   => 'Gtk2::CellRendererText',
+				static  => {},
+				dynamic => { markup => MODEL_COL_MODE },
+			},
+		},
+		{
+			title  => 'Last Modified',
+			renderer => {
+				class   => 'Gtk2::CellRendererText',
+				static  => {},
+				dynamic => { markup => MODEL_COL_MODIFIED },
+			},
+			sort_column => MODEL_COL_SORT_MODIFIED,
+		}
+	);
+	
+	$self->get_model->set_sort_column_id(MODEL_COL_SORT_NAME, 'ascending');
+	$self->set_search_column(MODEL_COL_NAME);
 	$self->show($inventory) if ($inventory);
 
 	return $self;
 }
 
-sub _init ($) {
-	my $self = shift;
-
-	$self->set_rules_hint(TRUE);
-	$self->set_search_column(MODEL_COL_NAME);
-
-	foreach my $col (@VIEW_COLUMNS) {
-		my $column = Gtk2::TreeViewColumn->new;
-		$column->set_title($col->{title});
-
-		foreach my $rdr ($col->{renderer} || @{$col->{renderers}}) {
-			my $renderer = Gtk2::Object->new(
-				$rdr->{class},
-				%{$rdr->{static}},
-			);
-
-			$column->pack_start($renderer, TRUE);
-			$column->set_attributes($renderer, %{$rdr->{dynamic}});
-		}
-
-		$column->set_resizable(TRUE);
-		$column->set_sort_column_id($col->{sort_column})
-			if exists $col->{sort_column};
-
-		$self->append_column($column);
-	}
-}
-
-sub get ($$;@) {
-	my $self = shift;
-	my $iter = shift;
-
-	return $self->get_model()->get($iter, @_);
-}
-
-sub get_selected ($;@) {
-	my $self = shift;
-
-	my $iter = $self->get_selection()->get_selected();
-
-	return $iter ? $self->get($iter, @_) : undef;
-}
-
 sub get_selected_path ($) {
 	my $self = shift;
 
-	return $self->get_selected(MODEL_COL_PATH);
+	return $self->get_selection->count_selected_rows
+		? $self->get_selected(MODEL_COL_ENTRY)->{path}
+		: undef;
 }
 
 sub select_by_path ($$) {
 	my $self = shift;
 	my $path = shift;
 
-	my $model = $self->get_model();
-	my $iter  = $model->get_iter_first();
-	while (defined $iter) {
-		my $ipath = $model->get($iter, MODEL_COL_PATH);
-
-		if ($ipath eq $path) {
-			$self->get_selection()->select_iter($iter);
-			last;
-		} else {
-			$iter = $model->iter_next($iter);
-		}
-	}
+	$self->select(sub { $_[0]->get($_[1], MODEL_COL_ENTRY)->{path} eq $path });
 }
 
 sub show ($$$) {
@@ -211,8 +160,8 @@ sub show ($$$) {
 	my $inventory = shift;
 	my $parent    = shift;
 
-	my $model = $self->get_model();
-	$model->clear();
+	my $model = $self->get_model;
+	$model->clear;
 
 	return unless (defined $inventory);
 
@@ -267,7 +216,10 @@ sub list_model_append ($$$) {
 		MODEL_COL_CATEGORY, $CATEGORY_NAMES{$entry->{category}},
 		MODEL_COL_NAME,     $name,
 		MODEL_COL_STATUS,   inventory_entry_status($entry),
-		MODEL_COL_PATH,     $entry->{path},
+
+		MODEL_COL_SORT_NAME, ($entry->{type} eq DIRECTORY ? 0 : 1) . $name,
+
+		MODEL_COL_ENTRY,    $entry,
 	);
 
 	if ($entry->{stat}) {
@@ -275,12 +227,18 @@ sub list_model_append ($$$) {
 			MODEL_COL_SIZE,     $entry->{stat}->[7],
 			MODEL_COL_MODE,     mode_string($entry->{stat}->[2]),
 			MODEL_COL_MODIFIED, strftime('%c', localtime($entry->{stat}->[9])),
+
+			MODEL_COL_SORT_SIZE,     $entry->{stat}->[7],
+			MODEL_COL_SORT_MODIFIED, $entry->{stat}->[9],
 		);
 	} else {
 		$model->set($iter,
 			MODEL_COL_SIZE,     '<i>unknown</i>',
 			MODEL_COL_MODE,     '<i>unknown</i>',
 			MODEL_COL_MODIFIED, '<i>unknown</i>',
+
+			MODEL_COL_SORT_SIZE,     0,
+			MODEL_COL_SORT_MODIFIED, 0,
 		);
 	}
 }
